@@ -1,6 +1,6 @@
 
 /*
- * $Id: HttpHeader.c,v 1.61.2.1 1999/02/12 19:38:18 wessels Exp $
+ * $Id: HttpHeader.c,v 1.63.2.3 1999/10/21 21:09:16 wessels Exp $
  *
  * DEBUG: section 55    HTTP Header
  * AUTHOR: Alex Rousskov
@@ -141,6 +141,7 @@ static http_hdr_type ListHeadersArr[] =
     HDR_CONNECTION,
     HDR_IF_MATCH, HDR_IF_NONE_MATCH,
     HDR_LINK, HDR_PRAGMA,
+    HDR_PROXY_CONNECTION,
     /* HDR_TRANSFER_ENCODING, */
     HDR_UPGRADE,
     HDR_VARY,
@@ -227,7 +228,7 @@ static void httpHeaderStatDump(const HttpHeaderStat * hs, StoreEntry * e);
  */
 
 void
-httpHeaderInitModule()
+httpHeaderInitModule(void)
 {
     int i;
     /* check that we have enough space for masks */
@@ -264,7 +265,7 @@ httpHeaderInitModule()
 }
 
 void
-httpHeaderCleanModule()
+httpHeaderCleanModule(void)
 {
     httpHeaderDestroyFieldsInfo(Headers, HDR_ENUM_END);
     Headers = NULL;
@@ -383,7 +384,16 @@ httpHeaderParse(HttpHeader * hdr, const char *header_start, const char *header_e
     HttpHeaderStats[hdr->owner].parsedCount++;
     /* commonn format headers are "<name>:[ws]<value>" lines delimited by <CRLF> */
     while (field_start < header_end) {
-	const char *field_end = field_start + strcspn(field_start, "\r\n");
+	const char *field_end;
+	const char *field_ptr = field_start;
+	do {
+	    field_end = field_ptr = field_ptr + strcspn(field_ptr, "\r\n");
+	    /* skip CRLF */
+	    if (*field_ptr == '\r')
+		field_ptr++;
+	    if (*field_ptr == '\n')
+		field_ptr++;
+	} while (*field_ptr == ' ' || *field_ptr == '\t');
 	if (!*field_end || field_end > header_end)
 	    return httpHeaderReset(hdr);	/* missing <CRLF> */
 	e = httpHeaderEntryParseCreate(field_start, field_end);
