@@ -1,6 +1,6 @@
 
 /*
- * $Id: acl.c,v 1.197.2.5 1999/04/19 00:14:33 wessels Exp $
+ * $Id: acl.c,v 1.205.2.13 2000/01/05 23:40:00 wessels Exp $
  *
  * DEBUG: section 28    Access Control
  * AUTHOR: Duane Wessels
@@ -175,6 +175,10 @@ aclStrToType(const char *s)
 	return ACL_URL_REGEX;
     if (!strcmp(s, "port"))
 	return ACL_URL_PORT;
+    if (!strcmp(s, "myport"))
+	return ACL_MY_PORT;
+    if (!strcmp(s, "maxconn"))
+	return ACL_MAXCONN;
 #if USE_IDENT
     if (!strcmp(s, "ident"))
 	return ACL_IDENT;
@@ -229,6 +233,10 @@ aclTypeToStr(squid_acl type)
 	return "url_regex";
     if (type == ACL_URL_PORT)
 	return "port";
+    if (type == ACL_MY_PORT)
+	return "myport";
+    if (type == ACL_MAXCONN)
+	return "maxconn";
 #if USE_IDENT
     if (type == ACL_IDENT)
 	return "ident";
@@ -693,16 +701,19 @@ aclParseAclLine(acl ** head)
 	break;
     case ACL_URL_REGEX:
     case ACL_URLPATH_REGEX:
+    case ACL_BROWSER:
     case ACL_SRC_DOM_REGEX:
     case ACL_DST_DOM_REGEX:
 	aclParseRegexList(&A->data);
 	break;
     case ACL_SRC_ASN:
+    case ACL_MAXCONN:
     case ACL_DST_ASN:
     case ACL_NETDB_SRC_RTT:
 	aclParseIntlist(&A->data);
 	break;
     case ACL_URL_PORT:
+    case ACL_MY_PORT:
 	aclParseIntRange(&A->data);
 	break;
 #if USE_IDENT
@@ -716,9 +727,6 @@ aclParseAclLine(acl ** head)
     case ACL_METHOD:
 	aclParseMethodList(&A->data);
 	break;
-    case ACL_BROWSER:
-	aclParseRegexList(&A->data);
-	break;
     case ACL_PROXY_AUTH:
 	aclParseWordList(&A->data);
 	if (!proxy_auth_cache) {
@@ -744,7 +752,7 @@ aclParseAclLine(acl ** head)
     }
     /*
      * Clear AclMatchedName from our temporary hack
-    */
+     */
     AclMatchedName = NULL;	/* ugly */
     if (!new_acl)
 	return;
@@ -995,20 +1003,27 @@ aclDecodeProxyAuth(const char *proxy_auth, char **user, char **password, char *b
 {
     char *sent_auth;
     char *cleartext;
-
     if (proxy_auth == NULL)
 	return 0;
-    if (strlen(proxy_auth) < SKIP_BASIC_SZ)
+    debug(28, 6) ("aclDecodeProxyAuth: header = '%s'\n", proxy_auth);
+    if (strncasecmp(proxy_auth, "Basic ", 6) != 0) {
+	debug(28, 1) ("aclDecodeProxyAuth: Unsupported proxy-auth sheme, '%s'\n", proxy_auth);
 	return 0;
-    proxy_auth += SKIP_BASIC_SZ;
-    sent_auth = xstrdup(proxy_auth);	/* username and password */
-    /* Trim trailing \n before decoding */
-    strtok(sent_auth, "\n");
+    }
+    proxy_auth += 6;		/* "Basic " */
     /* Trim leading whitespace before decoding */
     while (xisspace(*proxy_auth))
 	proxy_auth++;
+    sent_auth = xstrdup(proxy_auth);	/* username and password */
+    /* Trim trailing \n before decoding */
+    strtok(sent_auth, "\n");
     cleartext = uudecode(sent_auth);
     xfree(sent_auth);
+    /*
+     * Don't allow NL or CR in the credentials.
+     * Oezguer Kesim <oec@codeblau.de>
+     */
+    strtok(cleartext, "\r\n");
     debug(28, 6) ("aclDecodeProxyAuth: cleartext = '%s'\n", cleartext);
     xstrncpy(buf, cleartext, bufsize);
     xfree(cleartext);
@@ -1016,7 +1031,12 @@ aclDecodeProxyAuth(const char *proxy_auth, char **user, char **password, char *b
     if ((*password = strchr(*user, ':')) != NULL)
 	*(*password)++ = '\0';
     if (*password == NULL) {
-	debug(28, 1) ("aclDecodeProxyAuth: no password in proxy authorization header\n");
+	debug(28, 1) ("aclDecodeProxyAuth: no password in proxy authorization header '%s'\n", proxy_auth);
+	return 0;
+    }
+    if (**password == '\0') {
+	debug(28, 1) ("aclDecodeProxyAuth: Disallowing empty password,"
+	    "user is '%s'\n", *user);
 	return 0;
     }
     return 1;
@@ -1044,7 +1064,8 @@ aclMatchProxyAuth(wordlist * data, const char *proxy_auth, acl_proxy_auth_user *
     debug(28, 5) ("aclMatchProxyAuth: checking user '%s'\n", user);
 
     if (auth_user) {
-	/* This should be optimized to a boolean argument indicating that the
+	/*
+	 * This should be optimized to a boolean argument indicating that the
 	 * password is invalid, instead of passing full acl_proxy_auth_user
 	 * structures, and all messing with checklist->proxy_auth should
 	 * be restricted the functions that deal with the authenticator.
@@ -1058,8 +1079,11 @@ aclMatchProxyAuth(wordlist * data, const char *proxy_auth, acl_proxy_auth_user *
 	    debug(28, 4) ("aclMatchProxyAuth: authentication failed for user '%s'\n",
 		user);
 	    aclFreeProxyAuthUser(auth_user);
-	    /* copy username to request for logging on client-side unless ident
-	     * is known (do not override ident with false proxy auth names) */
+	    /*
+	     * copy username to request for logging on client-side
+	     * unless ident is known (do not override ident with
+	     * false proxy auth names)
+	     */
 	    if (!*checklist->request->user_ident)
 		xstrncpy(checklist->request->user_ident, user, USER_IDENT_SZ);
 	    return -2;
@@ -1069,6 +1093,8 @@ aclMatchProxyAuth(wordlist * data, const char *proxy_auth, acl_proxy_auth_user *
 	    /* store validated user in hash, after filling in expiretime */
 	    xstrncpy(checklist->request->user_ident, user, USER_IDENT_SZ);
 	    auth_user->expiretime = current_time.tv_sec + Config.authenticateTTL;
+	    auth_user->ip_expiretime = squid_curtime + Config.authenticateIpTTL;
+	    auth_user->ipaddr = checklist->src_addr;
 	    hash_join(proxy_auth_cache, (hash_link *) auth_user);
 	    /* Continue checking below, as normal */
 	}
@@ -1082,12 +1108,26 @@ aclMatchProxyAuth(wordlist * data, const char *proxy_auth, acl_proxy_auth_user *
 	return -1;
     } else if ((0 == strcmp(auth_user->passwd, password)) &&
 	(auth_user->expiretime > current_time.tv_sec)) {
-	/* user already known and valid */
-	debug(28, 5) ("aclMatchProxyAuth: user '%s' previously validated\n",
-	    user);
-	/* copy username to request for logging on client-side */
-	xstrncpy(checklist->request->user_ident, user, USER_IDENT_SZ);
-	return aclMatchUser(data, user);
+	if (checklist->src_addr.s_addr == auth_user->ipaddr.s_addr
+	    || auth_user->ip_expiretime <= squid_curtime) {
+	    /* user already known and valid */
+	    debug(28, 5) ("aclMatchProxyAuth: user '%s' previously validated\n",
+		user);
+	    /* Update IP ttl */
+	    auth_user->ip_expiretime = squid_curtime + Config.authenticateIpTTL;
+	    auth_user->ipaddr = checklist->src_addr;
+	    /* copy username to request for logging on client-side */
+	    xstrncpy(checklist->request->user_ident, user, USER_IDENT_SZ);
+	    return aclMatchUser(data, user);
+	} else {
+	    /* user has switched to another IP addr */
+	    debug(28, 1) ("aclMatchProxyAuth: user '%s' has changed IP address\n", user);
+	    /* remove this user from the hash, making him unknown */
+	    hash_remove_link(proxy_auth_cache, (hash_link *) auth_user);
+	    aclFreeProxyAuthUser(auth_user);
+	    /* require the user to reauthenticate */
+	    return -2;
+	}
     } else {
 	/* password mismatch/timeout */
 	debug(28, 4) ("aclMatchProxyAuth: user '%s' password mismatch/timeout\n",
@@ -1234,6 +1274,7 @@ aclMatchAcl(acl * ae, aclCheck_t * checklist)
     const char *fqdn = NULL;
     char *esc_buf;
     const char *header;
+    const char *browser;
     int k;
     if (!ae)
 	return 0;
@@ -1331,8 +1372,15 @@ aclMatchAcl(acl * ae, aclCheck_t * checklist)
 	safe_free(esc_buf);
 	return k;
 	/* NOTREACHED */
+    case ACL_MAXCONN:
+	k = clientdbEstablished(checklist->src_addr, 0);
+	return ((k > ((intlist *) ae->data)->i) ? 0 : 1);
+	/* NOTREACHED */
     case ACL_URL_PORT:
-	return aclMatchIntegerRange(ae->data, r->port);
+	return aclMatchIntegerRange(ae->data, (int) r->port);
+	/* NOTREACHED */
+    case ACL_MY_PORT:
+	return aclMatchIntegerRange(ae->data, (int) checklist->my_port);
 	/* NOTREACHED */
 #if USE_IDENT
     case ACL_IDENT:
@@ -1351,10 +1399,15 @@ aclMatchAcl(acl * ae, aclCheck_t * checklist)
 	return aclMatchInteger(ae->data, r->method);
 	/* NOTREACHED */
     case ACL_BROWSER:
-	return aclMatchRegex(ae->data, checklist->browser);
+	browser = httpHeaderGetStr(&checklist->request->header, HDR_USER_AGENT);
+	if (NULL == browser)
+	    return 0;
+	return aclMatchRegex(ae->data, browser);
 	/* NOTREACHED */
     case ACL_PROXY_AUTH:
-	if (!r->flags.accelerated) {
+	if (NULL == r) {
+	    return -1;
+	} else if (!r->flags.accelerated) {
 	    /* Proxy authorization on proxy requests */
 	    header = httpHeaderGetStr(&checklist->request->header,
 		HDR_PROXY_AUTHORIZATION);
@@ -1675,9 +1728,6 @@ aclLookupProxyAuthDone(void *data, char *result)
 aclCheck_t *
 aclChecklistCreate(const acl_access * A,
     request_t * request,
-    struct in_addr src_addr,
-    struct in_addr my_addr,
-    const char *user_agent,
     const char *ident)
 {
     int i;
@@ -1689,14 +1739,14 @@ aclChecklistCreate(const acl_access * A,
      * pointer, so lock it.
      */
     cbdataLock(A);
-    if (request != NULL)
+    if (request != NULL) {
 	checklist->request = requestLink(request);
-    checklist->src_addr = src_addr;
-    checklist->my_addr = my_addr;
+	checklist->src_addr = request->client_addr;
+	checklist->my_addr = request->my_addr;
+	checklist->my_port = request->my_port;
+    }
     for (i = 0; i < ACL_ENUM_MAX; i++)
 	checklist->state[i] = ACL_LOOKUP_NONE;
-    if (user_agent)
-	xstrncpy(checklist->browser, user_agent, BROWSERNAMELEN);
 #if USE_IDENT
     if (ident)
 	xstrncpy(checklist->ident, ident, USER_IDENT_SZ);
@@ -1795,20 +1845,25 @@ aclDestroyAcls(acl ** head)
 	case ACL_URL_REGEX:
 	case ACL_URLPATH_REGEX:
 	case ACL_BROWSER:
+	case ACL_SRC_DOM_REGEX:
+	case ACL_DST_DOM_REGEX:
 	    aclDestroyRegexList(a->data);
 	    break;
 	case ACL_PROTO:
 	case ACL_METHOD:
 	case ACL_SRC_ASN:
 	case ACL_DST_ASN:
+	case ACL_NETDB_SRC_RTT:
+	case ACL_MAXCONN:
 	    intlistDestroy((intlist **) & a->data);
 	    break;
 	case ACL_URL_PORT:
+	case ACL_MY_PORT:
 	    aclDestroyIntRange(a->data);
 	    break;
 	case ACL_NONE:
 	default:
-	    assert(0);
+	    debug(28, 1) ("aclDestroyAcls: no case for ACL type %d\n", a->type);
 	    break;
 	}
 	safe_free(a->cfgline);
@@ -1883,25 +1938,35 @@ aclDestroyIntRange(intrange * list)
 /* compare two domains */
 
 static int
-aclDomainCompare(const void *data, splayNode * n)
+aclDomainCompare(const void *a, const void *b)
 {
-    const char *d1 = data;
-    const char *d2 = n->data;
-    int l1 = strlen(d1);
-    int l2 = strlen(d2);
-    while (d1[l1] == d2[l2]) {
+    const char *d1 = a;
+    const char *d2 = b;
+    int l1;
+    int l2;
+    while ('.' == *d1)
+	d1++;
+    while ('.' == *d2)
+	d2++;
+    l1 = strlen(d1);
+    l2 = strlen(d2);
+    while (d1[--l1] == d2[--l2]) {
 	if ((l1 == 0) && (l2 == 0))
 	    return 0;		/* d1 == d2 */
-	if (l1-- == 0) {
-	    debug(28, 0) ("WARNING: %s is a subdomain of %s\n", d1, d2);
-	    debug(28, 0) ("WARNING: This may break Splay tree searching\n");
-	    debug(28, 0) ("WARNING: You should remove '%s' from the ACL named '%s'\n", d2, AclMatchedName);
+	if (0 == l1) {
+	    if ('.' == d2[l2 - 1]) {
+		debug(28, 0) ("WARNING: %s is a subdomain of %s\n", d2, d1);
+		debug(28, 0) ("WARNING: This may break Splay tree searching\n");
+		debug(28, 0) ("WARNING: You should remove '%s' from the ACL named '%s'\n", d2, AclMatchedName);
+	    }
 	    return -1;		/* d1 < d2 */
 	}
-	if (l2-- == 0) {
-	    debug(28, 0) ("WARNING: %s is a subdomain of %s\n", d2, d1);
-	    debug(28, 0) ("WARNING: This may break Splay tree searching\n");
-	    debug(28, 0) ("WARNING: You should remove '%s' from the ACL named '%s'\n", d1, AclMatchedName);
+	if (0 == l2) {
+	    if ('.' == d1[l1 - 1]) {
+		debug(28, 0) ("WARNING: %s is a subdomain of %s\n", d1, d2);
+		debug(28, 0) ("WARNING: This may break Splay tree searching\n");
+		debug(28, 0) ("WARNING: You should remove '%s' from the ACL named '%s'\n", d1, AclMatchedName);
+	    }
 	    return 1;		/* d1 > d2 */
 	}
     }
@@ -1911,31 +1976,11 @@ aclDomainCompare(const void *data, splayNode * n)
 /* compare a host and a domain */
 
 static int
-aclHostDomainCompare(const void *data, splayNode * n)
+aclHostDomainCompare(const void *a, const void *b)
 {
-    const char *h = data;
-    char *d = n->data;
-    int l1;
-    int l2;
-    if (matchDomainName(d, h))
-	return 0;
-    l1 = strlen(h);
-    l2 = strlen(d);
-    /* h != d */
-    while (xtolower(h[l1]) == xtolower(d[l2])) {
-	if (l1 == 0)
-	    break;
-	if (l2 == 0)
-	    break;
-	l1--;
-	l2--;
-    }
-    /* a '.' is a special case */
-    if ((h[l1] == '.') || (l1 == 0))
-	return -1;		/* domain(h) < d */
-    if ((d[l2] == '.') || (l2 == 0))
-	return 1;		/* domain(h) > d */
-    return (xtolower(h[l1]) - xtolower(d[l2]));
+    const char *h = a;
+    const char *d = b;
+    return matchDomainName(h, d);
 }
 
 /* compare two network specs
@@ -1952,12 +1997,12 @@ aclHostDomainCompare(const void *data, splayNode * n)
 /* compare an address and a network spec */
 
 static int
-aclIpNetworkCompare(const void *a, splayNode * n)
+aclIpNetworkCompare(const void *a, const void *b)
 {
-    struct in_addr A = *(struct in_addr *) a;
-    acl_ip_data *q = n->data;
-    struct in_addr B = q->addr1;
-    struct in_addr C = q->addr2;
+    struct in_addr A = *(const struct in_addr *) a;
+    const acl_ip_data *q = b;
+    const struct in_addr B = q->addr1;
+    const struct in_addr C = q->addr2;
     int rc = 0;
     A.s_addr &= q->mask.s_addr;	/* apply netmask */
     if (C.s_addr == 0) {	/* single address check */
@@ -2132,13 +2177,17 @@ aclDumpGeneric(const acl * a)
     case ACL_URL_REGEX:
     case ACL_URLPATH_REGEX:
     case ACL_BROWSER:
+    case ACL_SRC_DOM_REGEX:
+    case ACL_DST_DOM_REGEX:
 	return aclDumpRegexList(a->data);
 	break;
     case ACL_SRC_ASN:
+    case ACL_MAXCONN:
     case ACL_DST_ASN:
 	return aclDumpIntlistList(a->data);
 	break;
     case ACL_URL_PORT:
+    case ACL_MY_PORT:
 	return aclDumpIntRangeList(a->data);
 	break;
     case ACL_PROTO:
@@ -2154,6 +2203,7 @@ aclDumpGeneric(const acl * a)
 #endif
     case ACL_NONE:
     default:
+	debug(28, 1) ("aclDumpGeneric: no case for ACL type %d\n", a->type);
 	break;
     }
     return NULL;
@@ -2267,34 +2317,129 @@ aclMatchArp(void *dataptr, struct in_addr c)
 {
     struct arpreq arpReq;
     struct sockaddr_in ipAddr;
+    unsigned char ifbuffer[sizeof(struct ifreq) * 64];
+    struct ifconf ifc;
+    struct ifreq *ifr;
+    int offset;
     splayNode **Top = dataptr;
+    /*
+     * The linux kernel 2.2 maintains per interface ARP caches and
+     * thus requires an interface name when doing ARP queries.
+     * 
+     * The older 2.0 kernels appear to use a unified ARP cache,
+     * and require an empty interface name
+     * 
+     * To support both, we attempt the lookup with a blank interface
+     * name first. If that does not succeed, the try each interface
+     * in turn
+     */
+    /*
+     * Set up structures for ARP lookup with blank interface name
+     */
     ipAddr.sin_family = AF_INET;
     ipAddr.sin_port = 0;
     ipAddr.sin_addr = c;
+    memset(&arpReq, '\0', sizeof(arpReq));
     memcpy(&arpReq.arp_pa, &ipAddr, sizeof(struct sockaddr_in));
-    arpReq.arp_dev[0] = '\0';
-    arpReq.arp_flags = 0;
-    /* any AF_INET socket will do... gives back hardware type, device, etc */
-    if (ioctl(HttpSockets[0], SIOCGARP, &arpReq) == -1) {
-	debug(28, 1) ("ARP query failed - %d", errno);
-	return 0;
-    } else if (arpReq.arp_ha.sa_family != ARPHRD_ETHER) {
-	debug(28, 1) ("Non-ethernet interface returned from ARP query - %d",
-	    arpReq.arp_ha.sa_family);
-	/* update here and MAC address parsing to handle non-ethernet */
-	return 0;
-    } else
+    /* Query ARP table */
+    if (ioctl(HttpSockets[0], SIOCGARP, &arpReq) != -1) {
+	/* Skip non-ethernet interfaces */
+	if (arpReq.arp_ha.sa_family != ARPHRD_ETHER) {
+	    return 0;
+	}
+	debug(28, 4) ("Got address %02x:%02x:%02x:%02x:%02x:%02x\n",
+	    arpReq.arp_ha.sa_data[0] & 0xff, arpReq.arp_ha.sa_data[1] & 0xff,
+	    arpReq.arp_ha.sa_data[2] & 0xff, arpReq.arp_ha.sa_data[3] & 0xff,
+	    arpReq.arp_ha.sa_data[4] & 0xff, arpReq.arp_ha.sa_data[5] & 0xff);
+	/* Do lookup */
 	*Top = splay_splay(&arpReq.arp_ha.sa_data, *Top, aclArpCompare);
-    debug(28, 3) ("aclMatchArp: '%s' %s\n",
-	inet_ntoa(c), splayLastResult ? "NOT found" : "found");
-    return !splayLastResult;
+	debug(28, 3) ("aclMatchArp: '%s' %s\n",
+	    inet_ntoa(c), splayLastResult ? "NOT found" : "found");
+	return (0 == splayLastResult);
+    }
+    /* lookup list of interface names */
+    ifc.ifc_len = sizeof(ifbuffer);
+    ifc.ifc_buf = ifbuffer;
+    if (ioctl(HttpSockets[0], SIOCGIFCONF, &ifc) < 0) {
+	debug(28, 1) ("Attempt to retrieve interface list failed: %s\n",
+	    xstrerror());
+	return 0;
+    }
+    if (ifc.ifc_len > sizeof(ifbuffer)) {
+	debug(28, 1) ("Interface list too long - %d\n", ifc.ifc_len);
+	return 0;
+    }
+    /* Attempt ARP lookup on each interface */
+    offset = 0;
+    while (offset < ifc.ifc_len) {
+	ifr = (struct ifreq *) (ifbuffer + offset);
+	offset += sizeof(*ifr);
+	/* Skip loopback and aliased interfaces */
+	if (0 == strncmp(ifr->ifr_name, "lo", 2))
+	    continue;
+	if (NULL != strchr(ifr->ifr_name, ':'))
+	    continue;
+	debug(28, 4) ("Looking up ARP address for %s on %s\n", inet_ntoa(c),
+	    ifr->ifr_name);
+	/* Set up structures for ARP lookup */
+	ipAddr.sin_family = AF_INET;
+	ipAddr.sin_port = 0;
+	ipAddr.sin_addr = c;
+	memset(&arpReq, '\0', sizeof(arpReq));
+	memcpy(&arpReq.arp_pa, &ipAddr, sizeof(struct sockaddr_in));
+	strncpy(arpReq.arp_dev, ifr->ifr_name, sizeof(arpReq.arp_dev) - 1);
+	arpReq.arp_dev[sizeof(arpReq.arp_dev) - 1] = '\0';
+	/* Query ARP table */
+	if (-1 == ioctl(HttpSockets[0], SIOCGARP, &arpReq)) {
+	    /*
+	     * Query failed.  Do not log failed lookups or "device
+	     * not supported"
+	     */
+	    if (ENXIO == errno)
+		(void) 0;
+	    else if (ENODEV == errno)
+		(void) 0;
+	    else
+		debug(28, 1) ("ARP query failed: %s: %s\n",
+		    ifr->ifr_name, xstrerror());
+	    continue;
+	}
+	/* Skip non-ethernet interfaces */
+	if (arpReq.arp_ha.sa_family != ARPHRD_ETHER)
+	    continue;
+	debug(28, 4) ("Got address %02x:%02x:%02x:%02x:%02x:%02x on %s\n",
+	    arpReq.arp_ha.sa_data[0] & 0xff,
+	    arpReq.arp_ha.sa_data[1] & 0xff,
+	    arpReq.arp_ha.sa_data[2] & 0xff,
+	    arpReq.arp_ha.sa_data[3] & 0xff,
+	    arpReq.arp_ha.sa_data[4] & 0xff,
+	    arpReq.arp_ha.sa_data[5] & 0xff,
+	    ifr->ifr_name);
+	/* Do lookup */
+	*Top = splay_splay(&arpReq.arp_ha.sa_data, *Top, aclArpCompare);
+	/* Return if match, otherwise continue to other interfaces */
+	if (0 == splayLastResult) {
+	    debug(28, 3) ("aclMatchArp: %s found on %s\n",
+		inet_ntoa(c), ifr->ifr_name);
+	    return 1;
+	}
+	/*
+	 * Should we stop looking here? Can the same IP address
+	 * exist on multiple interfaces?
+	 */
+    }
+    /*
+     * Address was not found on any interface
+     */
+    debug(28, 3) ("aclMatchArp: %s NOT found\n", inet_ntoa(c));
+    return 0;
 }
 
 static int
-aclArpCompare(const void *data, splayNode * n)
+aclArpCompare(const void *a, const void *b)
 {
-    const unsigned short *d1 = data;
-    const unsigned short *d2 = n->data;
+    const unsigned short *d1 = a;
+    const unsigned short *d2 = b;
     if (d1[0] != d2[0])
 	return (d1[0] > d2[0]) ? 1 : -1;
     if (d1[1] != d2[1])
@@ -2373,7 +2518,7 @@ aclDumpArpListWalkee(void *node, void *state)
     static char buf[24];
     while (*W != NULL)
 	W = &(*W)->next;
-    snprintf(buf, sizeof(buf), "%02x:%02x:02x:02x:02x:02x",
+    snprintf(buf, sizeof(buf), "%02x:%02x:%02x:%02x:%02x:%02x",
 	arp->eth[0], arp->eth[1], arp->eth[2], arp->eth[3],
 	arp->eth[4], arp->eth[5]);
     wordlistAdd(state, buf);
