@@ -1,6 +1,6 @@
 
 /*
- * $Id: cachemgr.c,v 1.84.2.2 1999/02/12 19:38:23 wessels Exp $
+ * $Id: cachemgr.c,v 1.86 1999/04/19 04:42:37 wessels Exp $
  *
  * DEBUG: section 0     CGI Cache Manager
  * AUTHOR: Duane Wessels
@@ -239,7 +239,7 @@ auth_html(char *host, int port, const char *user_name)
     printf("for the Squid object cache.</P>\n");
     printf("<HR>\n");
     printf("<FORM METHOD=\"GET\" ACTION=\"%s\">\n", script_name);
-    printf("<TABLE BORDER=0>\n");
+    printf("<TABLE BORDER=\"0\" CELLPADDING=\"10\" CELLSPACING=\"1\">\n");
     printf("<TR><TH ALIGN=\"left\">Cache Host:</TH><TD><INPUT NAME=\"host\" ");
     printf("SIZE=30 VALUE=\"%s\"></TD></TR>\n", host);
     printf("<TR><TH ALIGN=\"left\">Cache Port:</TH><TD><INPUT NAME=\"port\" ");
