@@ -1,6 +1,6 @@
 
 /*
- * $Id: comm.c,v 1.299 1999/01/18 22:23:32 wessels Exp $
+ * $Id: comm.c,v 1.302.2.2 1999/10/22 20:08:06 wessels Exp $
  *
  * DEBUG: section 5     Socket Functions
  * AUTHOR: Harvest Derived
@@ -39,12 +39,6 @@
 #include <netinet/tcp.h>
 #endif
 
-#if USE_ASYNC_IO
-#define MAX_POLL_TIME 10
-#else
-#define MAX_POLL_TIME 1000
-#endif
-
 typedef struct {
     char *host;
     u_short port;
@@ -166,11 +160,11 @@ comm_open(int sock_type,
 	case ENFILE:
 	case EMFILE:
 	    debug(50, 1) ("comm_open: socket failure: %s\n", xstrerror());
+	    fdAdjustReserved();
 	    break;
 	default:
 	    debug(50, 0) ("comm_open: socket failure: %s\n", xstrerror());
 	}
-	fdAdjustReserved();
 	return -1;
     }
     /* update fdstat */
@@ -308,12 +302,14 @@ commResetFD(ConnectStateData * cs)
     Counter.syscalls.sock.sockets++;
     if (fd2 < 0) {
 	debug(5, 0) ("commResetFD: socket: %s\n", xstrerror());
-	fdAdjustReserved();
+	if (ENFILE == errno || EMFILE == errno)
+	    fdAdjustReserved();
 	return 0;
     }
     if (dup2(fd2, cs->fd) < 0) {
 	debug(5, 0) ("commResetFD: dup2: %s\n", xstrerror());
-	fdAdjustReserved();
+	if (ENFILE == errno || EMFILE == errno)
+	    fdAdjustReserved();
 	return 0;
     }
     close(fd2);
@@ -325,7 +321,6 @@ commResetFD(ConnectStateData * cs)
     commSetCloseOnExec(cs->fd);
     if (Config.Addrs.tcp_outgoing.s_addr != no_addr.s_addr) {
 	if (commBind(cs->fd, Config.Addrs.tcp_outgoing, 0) != COMM_OK) {
-	    comm_close(cs->fd);
 	    return 0;
 	}
     }
@@ -570,9 +565,6 @@ void
 comm_close(int fd)
 {
     fde *F = NULL;
-#if USE_ASYNC_IO
-    int doaioclose = 1;
-#endif
     debug(5, 5) ("comm_close: FD %d\n", fd);
     assert(fd >= 0);
     assert(fd < Squid_MaxFD);
@@ -583,30 +575,13 @@ comm_close(int fd)
 	return;
     assert(F->flags.open);
     assert(F->type != FD_FILE);
-#ifdef USE_ASYNC_IO
-    if (F->flags.nolinger && F->flags.nonblocking)
-	doaioclose = 0;
-#endif
     F->flags.closing = 1;
     CommWriteStateCallbackAndFree(fd, COMM_ERR_CLOSING);
     commCallCloseHandlers(fd);
     if (F->uses)		/* assume persistent connect count */
 	pconnHistCount(1, F->uses);
     fd_close(fd);		/* update fdstat */
-#if defined(_SQUID_LINUX_)
-    /*
-     * michael@metal.iinet.net.au sez close() on
-     * network sockets never blocks.
-     */
     close(fd);
-#elif USE_ASYNC_IO
-    if (doaioclose)
-	aioClose(fd);
-    else
-	close(fd);
-#else
-    close(fd);
-#endif
     Counter.syscalls.sock.closes++;
 }
 
@@ -745,6 +720,23 @@ commSetNonBlocking(int fd)
     return 0;
 }
 
+int
+commUnsetNonBlocking(int fd)
+{
+    int flags;
+    int dummy = 0;
+    if ((flags = fcntl(fd, F_GETFL, dummy)) < 0) {
+	debug(50, 0) ("FD %d: fcntl F_GETFL: %s\n", fd, xstrerror());
+	return COMM_ERROR;
+    }
+    if (fcntl(fd, F_SETFL, flags & (~SQUID_NONBLOCK)) < 0) {
+	debug(50, 0) ("commUnsetNonBlocking: FD %d: %s\n", fd, xstrerror());
+	return COMM_ERROR;
+    }
+    fd_table[fd].flags.nonblocking = 0;
+    return 0;
+}
+
 void
 commSetCloseOnExec(int fd)
 {
@@ -864,11 +856,7 @@ comm_write(int fd, char *buf, int size, CWCB * handler, void *handler_data, FREE
     state->handler_data = handler_data;
     state->free_func = free_func;
     cbdataLock(handler_data);
-#ifdef OPTIMISTIC_IO
-    commHandleWrite(fd, state);
-#else
     commSetSelect(fd, COMM_SELECT_WRITE, commHandleWrite, state, 0);
-#endif
 }
 
 /* a wrapper around comm_write to allow for MemBuf to be comm_written in a snap */
