@@ -1,6 +1,6 @@
 
 /*
- * $Id: delay_pools.c,v 1.7.2.7 1999/04/12 17:25:13 wessels Exp $
+ * $Id: delay_pools.c,v 1.10.2.2 1999/10/05 22:29:41 wessels Exp $
  *
  * DEBUG: section 77    Delay Pools
  * AUTHOR: David Luyer <luyer@ucs.uwa.edu.au>
@@ -113,8 +113,12 @@ delayIdPtrHash(const void *key, unsigned int n)
 static int
 delayIdPtrHashCmp(const void *a, const void *b)
 {
-    /* Sort by POINTER VALUE. */
-    return b - a;
+    /*
+     * Compare POINTER VALUE.
+     * Note, we can't subtract void pointers, but we don't need
+     * to anyway.  All we need is a test for equality.
+     */
+    return a != b;
 }
 
 void
@@ -145,7 +149,7 @@ delayIdZero(void *hlink)
 }
 
 void
-delayFreeDelayData()
+delayFreeDelayData(void)
 {
     safe_free(delay_data);
     if (!delay_id_ptr_hash)
@@ -161,6 +165,8 @@ delayRegisterDelayIdPtr(delay_id * loc)
     hash_link *lnk;
     if (!delay_id_ptr_hash)
 	return;
+    if (*loc == 0)
+	return;
     lnk = xmalloc(sizeof(hash_link));
     lnk->key = (char *) loc;
     hash_join(delay_id_ptr_hash, lnk);
@@ -277,6 +283,7 @@ delayClient(request_t * r)
     memset(&ch, '\0', sizeof(ch));
     ch.src_addr = r->client_addr;
     ch.my_addr = r->my_addr;
+    ch.my_port = r->my_port;
     ch.request = r;
     for (pool = 0; pool < Config.Delay.pools; pool++) {
 	if (aclCheckFast(Config.Delay.access[pool], &ch))
@@ -623,6 +630,7 @@ delaySetStoreClient(StoreEntry * e, void *data, delay_id delay_id)
     store_client *sc = storeClientListSearch(e->mem_obj, data);
     assert(sc != NULL);
     sc->delay_id = delay_id;
+    delayRegisterDelayIdPtr(&sc->delay_id);
 }
 
 static void
