@@ -1,6 +1,6 @@
 
 /*
- * $Id: main.c,v 1.290.2.1 1999/02/19 22:35:35 wessels Exp $
+ * $Id: main.c,v 1.302.2.13 2000/01/05 23:45:31 wessels Exp $
  *
  * DEBUG: section 1     Startup and Main Loop
  * AUTHOR: Harvest Derived
@@ -72,6 +72,10 @@ extern void log_trace_init(char *);
 static EVH SquidShutdown;
 static void mainSetCwd(void);
 
+#if TEST_ACCESS
+#include "test_access.c"
+#endif
+
 static void
 usage(void)
 {
@@ -91,7 +95,7 @@ usage(void)
 	"       -z        Create swap directories\n"
 	"       -C        Do not catch fatal signals.\n"
 	"       -D        Disable initial DNS tests.\n"
-	"       -F        Foreground fast store rebuild.\n"
+	"       -F        Don't serve any requests until store is rebuilt.\n"
 	"       -N        No daemon mode.\n"
 	"       -R        Do not set REUSEADDR on port.\n"
 	"       -V        Virtual host httpd-accelerator.\n"
@@ -279,6 +283,9 @@ serverConnectionsOpen(void)
 #endif
 #ifdef SQUID_SNMP
     snmpConnectionOpen();
+#endif
+#if USE_WCCP
+    wccpConnectionOpen();
 #endif
     clientdbInit();
     icmpOpen();
@@ -302,6 +309,9 @@ serverConnectionsClose(void)
     icmpClose();
 #ifdef SQUID_SNMP
     snmpConnectionShutdown();
+#endif
+#if USE_WCCP
+    wccpConnectionShutdown();
 #endif
     asnFreeMemory();
 }
@@ -319,20 +329,33 @@ mainReconfigure(void)
 #endif
 #ifdef SQUID_SNMP
     snmpConnectionClose();
+#endif
+#if USE_WCCP
+    wccpConnectionClose();
 #endif
     dnsShutdown();
+#if !USE_DNSSERVERS
+    idnsShutdown();
+#endif
     redirectShutdown();
     authenticateShutdown();
     storeDirCloseSwapLogs();
     errorClean();
+    mimeFreeMemory();
     parseConfigFile(ConfigFile);
     _db_init(Config.Log.log, Config.debugOptions);
     ipcache_restart();		/* clear stuck entries */
     fqdncache_restart();	/* sigh, fqdncache too */
     errorInitialize();		/* reload error pages */
     dnsInit();
+#if !USE_DNSSERVERS
+    idnsInit();
+#endif
     redirectInit();
     authenticateInit();
+#if USE_WCCP
+    wccpInit();
+#endif
     serverConnectionsOpen();
     if (theOutIcpConnection >= 0) {
 	if (!Config2.Accel.on || Config.onoff.accel_with_proxy)
@@ -341,6 +364,7 @@ mainReconfigure(void)
 	    debug(1, 1) ("ICP port disabled in httpd_accelerator mode\n");
     }
     storeDirOpenSwapLogs();
+    mimeInit(Config.mimeTablePathname);
     writePidFile();		/* write PID file */
     debug(1, 1) ("Ready to serve requests.\n");
     reconfiguring = 0;
@@ -405,6 +429,10 @@ mainSetCwd(void)
 static void
 mainInitialize(void)
 {
+    /* chroot if configured to run inside chroot */
+    if (Config.chroot_dir && chroot(Config.chroot_dir)) {
+	fatal("failed to chroot");
+    }
     if (opt_catch_signals) {
 	squid_signal(SIGSEGV, death, SA_NODEFER | SA_RESETHAND);
 	squid_signal(SIGBUS, death, SA_NODEFER | SA_RESETHAND);
@@ -413,9 +441,9 @@ mainInitialize(void)
     squid_signal(SIGCHLD, sig_child, SA_NODEFER | SA_RESTART);
 
     setEffectiveUser();
-    assert(Config.Port.http);
+    assert(Config.Sockaddr.http);
     if (httpPortNumOverride != 1)
-	Config.Port.http->i = (u_short) httpPortNumOverride;
+	Config.Sockaddr.http->s.sin_port = htons(httpPortNumOverride);
     if (icpPortNumOverride != 1)
 	Config.Port.icp = (u_short) icpPortNumOverride;
 
@@ -435,6 +463,9 @@ mainInitialize(void)
     ipcache_init();
     fqdncache_init();
     dnsInit();
+#if !USE_DNSSERVERS
+    idnsInit();
+#endif
     redirectInit();
     authenticateInit();
     useragentOpenLog();
@@ -448,12 +479,14 @@ mainInitialize(void)
 #ifdef SQUID_SNMP
     snmpInit();
 #endif
-
 #if MALLOC_DBG
     malloc_debug(0, malloc_debug_level);
 #endif
 
     if (!configured_once) {
+#if USE_ASYNC_IO
+	aioInit();
+#endif
 	unlinkdInit();
 	urlInitialize();
 	cachemgrInit();
@@ -464,13 +497,15 @@ mainInitialize(void)
 	do_mallinfo = 1;
 	mimeInit(Config.mimeTablePathname);
 	pconnInit();
-	eventInit();
 	refreshInit();
 #if DELAY_POOLS
 	delayPoolsInit();
 #endif
 	fwdInit();
     }
+#if USE_WCCP
+    wccpInit();
+#endif
     serverConnectionsOpen();
     if (theOutIcpConnection >= 0) {
 	if (!Config2.Accel.on || Config.onoff.accel_with_proxy)
@@ -478,6 +513,8 @@ mainInitialize(void)
 	else
 	    debug(1, 1) ("ICP port disabled in httpd_accelerator mode\n");
     }
+    if (Config.chroot_dir)
+	no_suid();
     if (!configured_once)
 	writePidFile();		/* write PID file */
 
@@ -495,11 +532,10 @@ mainInitialize(void)
     squid_signal(SIGALRM, time_tick, SA_RESTART);
     alarm(1);
 #endif
+    memCheckInit();
     debug(1, 1) ("Ready to serve requests.\n");
-
     if (!configured_once) {
 	eventAdd("storeMaintain", storeMaintainSwapSpace, NULL, 1.0, 1);
-	eventAdd("storeDirClean", storeDirClean, NULL, 15.0, 1);
 	if (Config.onoff.announce)
 	    eventAdd("start_announce", start_announce, NULL, 3600.0, 1);
 	eventAdd("ipcache_purgelru", ipcache_purgelru, NULL, 10.0, 1);
@@ -514,6 +550,7 @@ main(int argc, char **argv)
     int errcount = 0;
     int n;			/* # of GC'd objects */
     time_t loop_delay;
+    mode_t oldmask;
 
     debug_log = stderr;
     if (FD_SETSIZE < Squid_MaxFD)
@@ -535,6 +572,16 @@ main(int argc, char **argv)
 #endif
 #endif /* HAVE_MALLOPT */
 
+    /*
+     * The plan here is to set the umask to 007 (deny others for
+     * read,write,execute), but only if the umask is not already
+     * set.  Unfortunately, there is no way to get the current
+     * umask value without setting it.
+     */
+    oldmask = umask(S_IRWXO);
+    if (oldmask)
+	umask(oldmask);
+
     memset(&local_addr, '\0', sizeof(struct in_addr));
     safe_inet_addr(localhost, &local_addr);
     memset(&any_addr, '\0', sizeof(struct in_addr));
@@ -561,18 +608,35 @@ main(int argc, char **argv)
 	leakInit();
 #endif
 	memInit();		/* memInit is required for config parsing */
+	eventInit();		/* eventInit() is required for config parsing */
 	parse_err = parseConfigFile(ConfigFile);
 
 	if (opt_parse_cfg_only)
 	    return parse_err;
     }
 
+#if TEST_ACCESS
+    comm_init();
+    comm_select_init();
+    mainInitialize();
+    test_access();
+    return 0;
+#endif
+
     /* send signal to running copy and exit */
     if (opt_send_signal != -1) {
+	/* chroot if configured to run inside chroot */
+	if (Config.chroot_dir && chroot(Config.chroot_dir)) {
+	    fatal("failed to chroot");
+	}
 	sendSignal();
 	/* NOTREACHED */
     }
     if (opt_create_swap_dirs) {
+	/* chroot if configured to run inside chroot */
+	if (Config.chroot_dir && chroot(Config.chroot_dir)) {
+	    fatal("failed to chroot");
+	}
 	setEffectiveUser();
 	debug(0, 0) ("Creating Swap Directories\n");
 	storeCreateSwapDirectories();
@@ -616,6 +680,9 @@ main(int argc, char **argv)
 	    shutting_down = 1;
 	    serverConnectionsClose();
 	    dnsShutdown();
+#if !USE_DNSSERVERS
+	    idnsShutdown();
+#endif
 	    redirectShutdown();
 	    authenticateShutdown();
 	    eventAdd("SquidShutdown", SquidShutdown, NULL, (double) (wait + 1), 1);
@@ -705,7 +772,6 @@ watch_child(char *argv[])
 #endif
     for (i = 0; i < Squid_MaxFD; i++)
 	close(i);
-    umask(0);
     for (;;) {
 	if ((pid = fork()) == 0) {
 	    /* child */
@@ -767,6 +833,9 @@ SquidShutdown(void *unused)
 #endif
 #ifdef SQUID_SNMP
     snmpConnectionClose();
+#endif
+#if USE_WCCP
+    wccpConnectionClose();
 #endif
     releaseServerSockets();
     commCloseAllSockets();
@@ -819,6 +888,7 @@ SquidShutdown(void *unused)
 #endif
     debug(1, 1) ("Squid Cache (Version %s): Exiting normally.\n",
 	version_string);
-    fclose(debug_log);
+    if (debug_log)
+	fclose(debug_log);
     exit(0);
 }
