@@ -1,6 +1,6 @@
 
 /*
- * $Id: peer_select.c,v 1.98 1999/01/29 23:01:07 wessels Exp $
+ * $Id: peer_select.c,v 1.100.2.4 2000/01/05 23:40:04 wessels Exp $
  *
  * DEBUG: section 44    Peer Selection Algorithm
  * AUTHOR: Duane Wessels
@@ -52,7 +52,8 @@ const char *hier_strings[] =
     "SOURCE_FASTEST",
     "ROUNDROBIN_PARENT",
 #if USE_CACHE_DIGESTS
-    "CACHE_DIGEST_HIT",
+    "CD_PARENT_HIT",
+    "CD_SIBLING_HIT",
 #endif
 #if USE_CARP
     "CARP",
@@ -236,9 +237,6 @@ peerSelectFoo(ps_state * ps)
 	    ps->acl_checklist = aclChecklistCreate(
 		Config.accessList.AlwaysDirect,
 		request,
-		request->client_addr,
-		request->my_addr,
-		NULL,		/* user agent */
 		NULL);		/* ident */
 	    aclNBCheck(ps->acl_checklist,
 		peerCheckAlwaysDirectDone,
@@ -250,9 +248,6 @@ peerSelectFoo(ps_state * ps)
 	    ps->acl_checklist = aclChecklistCreate(
 		Config.accessList.NeverDirect,
 		request,
-		request->client_addr,
-		request->my_addr,
-		NULL,		/* user agent */
 		NULL);		/* ident */
 	    aclNBCheck(ps->acl_checklist,
 		peerCheckNeverDirectDone,
@@ -310,7 +305,10 @@ peerGetSomeNeighbor(ps_state * ps)
     }
 #if USE_CACHE_DIGESTS
     if ((p = neighborsDigestSelect(request, entry))) {
-	code = CACHE_DIGEST_HIT;
+	if (neighborType(p, request) == PEER_PARENT)
+	    code = CD_PARENT_HIT;
+	else
+	    code = CD_SIBLING_HIT;
     } else
 #endif
 #if USE_CARP
@@ -420,6 +418,8 @@ peerGetSomeParent(ps_state * ps)
     debug(44, 3) ("peerGetSomeParent: %s %s\n",
 	RequestMethodStr[request->method],
 	request->host);
+    if (ps->direct == DIRECT_YES)
+	return;
     if ((p = getDefaultParent(request))) {
 	code = DEFAULT_PARENT;
     } else if ((p = getRoundRobinParent(request))) {
@@ -442,9 +442,9 @@ peerPingTimeout(void *data)
     StoreEntry *entry = psstate->entry;
     if (entry)
 	debug(44, 3) ("peerPingTimeout: '%s'\n", storeUrl(entry));
-    entry->ping_status = PING_TIMEOUT;
     if (!cbdataValid(psstate->callback_data)) {
 	/* request aborted */
+	entry->ping_status = PING_DONE;
 	cbdataUnlock(psstate->callback_data);
 	peerSelectStateFree(psstate);
 	return;
@@ -532,7 +532,6 @@ static void
 peerHandleHtcpReply(peer * p, peer_t type, htcpReplyData * htcp, void *data)
 {
     ps_state *psstate = data;
-    request_t *request = psstate->request;
     debug(44, 3) ("peerHandleIcpReply: %s %s\n",
 	htcp->hit ? "HIT" : "MISS",
 	storeUrl(psstate->entry));
