@@ -1,6 +1,6 @@
 
 /*
- * $Id: snmp_core.c,v 1.34.2.5 1999/04/08 20:19:18 wessels Exp $
+ * $Id: snmp_core.c,v 1.39.2.1 1999/09/11 18:37:37 wessels Exp $
  *
  * DEBUG: section 49    SNMP support
  * AUTHOR: Glenn Chisholm
@@ -200,31 +200,31 @@ snmpInit(void)
 						snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 3, SQ_PRF, PERF_PROTO, 1, 14),
 						    LEN_SQ_PRF + 3, snmp_prfProtoFn, static_Inst, 0),
 						snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 3, SQ_PRF, PERF_PROTO, 1, 15),
-						    LEN_SQ_PRF + 3, snmp_prfProtoFn, static_Inst, 0))),
-					snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 2, SQ_PRF, PERF_PROTO, 2),
-					    LEN_SQ_PRF + 2, NULL, NULL, 1,
-					    snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 3, SQ_PRF, PERF_PROTO, 2, 1),
-						LEN_SQ_PRF + 3, NULL, NULL, 10,
-						snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 1),
-						    LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
-						snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 2),
-						    LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
-						snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 3),
-						    LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
-						snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 4),
-						    LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
-						snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 5),
-						    LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
-						snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 6),
-						    LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
-						snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 7),
-						    LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
-						snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 8),
-						    LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
-						snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 9),
-						    LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
-						snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 10),
-						    LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0)))),
+						    LEN_SQ_PRF + 3, snmp_prfProtoFn, static_Inst, 0)),
+					    snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 2, SQ_PRF, PERF_PROTO, 2),
+						LEN_SQ_PRF + 2, NULL, NULL, 1,
+						snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 3, SQ_PRF, PERF_PROTO, 2, 1),
+						    LEN_SQ_PRF + 3, NULL, NULL, 10,
+						    snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 1),
+							LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
+						    snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 2),
+							LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
+						    snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 3),
+							LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
+						    snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 4),
+							LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
+						    snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 5),
+							LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
+						    snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 6),
+							LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
+						    snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 7),
+							LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
+						    snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 8),
+							LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
+						    snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 9),
+							LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0),
+						    snmpAddNode(snmpCreateOid(LEN_SQ_PRF + 4, SQ_PRF, PERF_PROTO, 2, 1, 10),
+							LEN_SQ_PRF + 4, snmp_prfProtoFn, time_Inst, 0))))),
 				    snmpAddNode(snmpCreateOid(LEN_SQ_NET, SQ_NET),
 					LEN_SQ_NET, NULL, NULL, 3,
 					snmpAddNode(snmpCreateOid(LEN_SQ_NET + 1, SQ_NET, NET_IP_CACHE),
@@ -449,6 +449,7 @@ snmpHandleUdp(int sock, void *not_used)
     commSetSelect(sock, COMM_SELECT_READ, snmpHandleUdp, NULL, 0);
     from_len = sizeof(struct sockaddr_in);
     memset(&from, '\0', from_len);
+    memset(buf, '\0', SNMP_REQUEST_SIZE);
 
     Counter.syscalls.sock.recvfroms++;
 
@@ -482,7 +483,7 @@ snmpHandleUdp(int sock, void *not_used)
 /*
  * Turn SNMP packet into a PDU, check available ACL's
  */
-void
+static void
 snmpDecodePacket(snmp_request_t * rq)
 {
     struct snmp_pdu *PDU;
@@ -518,7 +519,7 @@ snmpDecodePacket(snmp_request_t * rq)
 /*
  * Packet OK, ACL Check OK, Create reponse.
  */
-void
+static void
 snmpConstructReponse(snmp_request_t * rq)
 {
     struct snmp_session Session;
@@ -545,7 +546,7 @@ snmpConstructReponse(snmp_request_t * rq)
  * 
  * If configured forward any reponses which are not for this agent.
  */
-struct snmp_pdu *
+static struct snmp_pdu *
 snmpAgentResponse(struct snmp_pdu *PDU)
 {
     struct snmp_pdu *Answer = NULL;
@@ -631,7 +632,7 @@ snmpAgentResponse(struct snmp_pdu *PDU)
     return (Answer);
 }
 
-oid_ParseFn *
+static oid_ParseFn *
 snmpTreeGet(oid * Current, snint CurrentLen)
 {
     oid_ParseFn *Fn = NULL;
@@ -652,13 +653,13 @@ snmpTreeGet(oid * Current, snint CurrentLen)
 	    count++;
 	}
     }
-    if (mibTreeEntry)
+    if (mibTreeEntry && mibTreeEntry->parsefunction)
 	Fn = mibTreeEntry->parsefunction;
     debug(49, 5) ("snmpTreeGet: return\n");
     return (Fn);
 }
 
-oid_ParseFn *
+static oid_ParseFn *
 snmpTreeNext(oid * Current, snint CurrentLen, oid ** Next, snint * NextLen)
 {
     oid_ParseFn *Fn = NULL;
@@ -724,7 +725,7 @@ snmpTreeNext(oid * Current, snint CurrentLen, oid ** Next, snint * NextLen)
     return (Fn);
 }
 
-oid *
+static oid *
 static_Inst(oid * name, snint * len, mib_tree_entry * current, oid_ParseFn ** Fn)
 {
     oid *instance = NULL;
@@ -739,7 +740,7 @@ static_Inst(oid * name, snint * len, mib_tree_entry * current, oid_ParseFn ** Fn
     return (instance);
 }
 
-oid *
+static oid *
 time_Inst(oid * name, snint * len, mib_tree_entry * current, oid_ParseFn ** Fn)
 {
     oid *instance = NULL;
@@ -766,7 +767,7 @@ time_Inst(oid * name, snint * len, mib_tree_entry * current, oid_ParseFn ** Fn)
     return (instance);
 }
 
-oid *
+static oid *
 peer_Inst(oid * name, snint * len, mib_tree_entry * current, oid_ParseFn ** Fn)
 {
     oid *instance = NULL;
@@ -825,7 +826,7 @@ peer_Inst(oid * name, snint * len, mib_tree_entry * current, oid_ParseFn ** Fn)
     return (instance);
 }
 
-oid *
+static oid *
 client_Inst(oid * name, snint * len, mib_tree_entry * current, oid_ParseFn ** Fn)
 {
     oid *instance = NULL;
@@ -873,7 +874,7 @@ client_Inst(oid * name, snint * len, mib_tree_entry * current, oid_ParseFn ** Fn
 /* 
  * Returns a the sibling object in the tree
  */
-mib_tree_entry *
+static mib_tree_entry *
 snmpTreeSiblingEntry(oid entry, snint len, mib_tree_entry * current)
 {
     mib_tree_entry *next = NULL;
@@ -896,7 +897,7 @@ snmpTreeSiblingEntry(oid entry, snint len, mib_tree_entry * current)
 /* 
  * Returns the requested child object or NULL if it does not exist
  */
-mib_tree_entry *
+static mib_tree_entry *
 snmpTreeEntry(oid entry, snint len, mib_tree_entry * current)
 {
     mib_tree_entry *next = NULL;
@@ -914,7 +915,7 @@ snmpTreeEntry(oid entry, snint len, mib_tree_entry * current)
 /*
  * Adds a node to the MIB tree structure and adds the appropriate children
  */
-mib_tree_entry *
+static mib_tree_entry *
 #if STDC_HEADERS
 snmpAddNode(oid * name, int len, oid_ParseFn * parsefunction, instance_Fn * instancefunction, int children,...)
 #else
@@ -967,7 +968,7 @@ snmpAddNode(va_alist)
 /* 
  * Returns the list of parameters in an oid
  */
-oid *
+static oid *
 #if STDC_HEADERS
 snmpCreateOid(int length,...)
 #else
@@ -1001,7 +1002,7 @@ snmpCreateOid(va_alist)
 /*
  * Allocate space for, and copy, an OID.  Returns new oid.
  */
-oid *
+static oid *
 snmpOidDup(oid * A, snint ALen)
 {
     oid *Ans = xmalloc(sizeof(oid) * ALen);
